from zope import interface
from zope.component.interfaces import IObjectEvent, ObjectEvent


class IMultilanguageIDModifiedEvent(IObjectEvent):
    """ An Event dispatched if one or more translated IDs of an
        object changed
    """


class MultilanguageIDModifiedEvent(ObjectEvent):
    """ An Event dispatched if one or more translated IDs of an
        object changed
    """
    interface.implements(IMultilanguageIDModifiedEvent)


class IMultilanguageURLHandler(interface.Interface):
    """ Manages ID translations of containers
    """

    def set_translated_id(id, translated, lang):
        """ Sets the translated ID for the given language and ID
        """

    def remove_translated_ids(id, event=True):
        """ Removes all registered translated IDs for the given ID
        """

    def get_translated_ids(id):
        """ Iterator of lang, ID pairs of all available translated IDs for the given ID
        """

    def get_translated_id(id, lang, event=True):
        """ Returns a translated ID of the given object and in the given language
        """

    def get_actual_id(translated):
        """ Returns the actual ID of the object linked with the given translated ID, None otherwise
        """

    def get_object(id):
        """ Returns the object having the given translated ID if available, None otherwise
        """

    def get_langs(id):
        """ Returns the languages of the given translated ID if available
        """

    def check_id(self, id):
        """ Whether the given ID is not already in use
        """



import transaction

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.utils import safe_callable

class Update(BrowserView):

    def __call__(self):
        context = aq_inner(self.context)
        catalog = getToolByName(context, 'portal_catalog')
        results = []
        def updateTranslatedPaths(obj, path=None):
            if (base_hasattr(obj, 'indexObject') and
                safe_callable(obj.indexObject) and
                base_hasattr(obj, 'getPhysicalPath') and
                safe_callable(obj.getPhysicalPath)):
                try:
                    path = '/'.join(obj.getPhysicalPath())
                    catalog._catalog.updateTanslatedPaths(obj, path)
                    results.append('Updated %s' % path)
                    transaction.commit()
                except:
                    pass
        context.ZopeFindAndApply(context, search_sub=True, apply_func=updateTranslatedPaths)
        return u'Successfully updated the IDs of the following objects:\n'+('='*54)+'\n\n' + '\n'.join(results)



from BTrees.OOBTree import OOBTree
from Acquisition import aq_acquire

from zope.interface import implements
from zope.component import adapts
from zope.event import notify
from zope.publisher.interfaces.http import IHTTPRequest
from zope.annotation.interfaces import IAnnotatable, IAnnotations
from OFS.interfaces import ITraversable
from ZODB.POSException import ConflictError
from Products.CMFPlone.utils import safe_unicode
from Products.Archetypes.interfaces.base import IBaseObject

from raptus.multilanguagefields.interfaces import IMultilanguageField
from raptus.multilanguageurls.interfaces import IMultilanguageURLHandler, MultilanguageIDModifiedEvent

ANNOTATIONS_KEY = 'raptus.multilanguageurls.mapping'


class MultilanguageURLHandler(object):
    implements(IMultilanguageURLHandler)
    adapts(IAnnotatable)

    def __init__(self, context):
        self.context = context
        annotations = IAnnotations(context)
        if not ANNOTATIONS_KEY in annotations:
            annotations[ANNOTATIONS_KEY] = OOBTree()
        self.storage = annotations[ANNOTATIONS_KEY]

    def dispatchEvent(self, id):
        notify(MultilanguageIDModifiedEvent(self.context[id]))

    def set_translated_id(self, id, translated, lang):
        """ Sets the translated ID for the given language and ID
        """
        id, translated = safe_unicode(id), safe_unicode(translated)
        if not id in self.context:
            return
        if not lang in self.storage:
            self.storage[lang] = OOBTree()
        if not 'index' in self.storage:
            self.storage['index'] = OOBTree()
        if not id == translated and not id == self.get_actual_id(translated) and not self.check_id(translated):
            obj = self.context[id]
            translated = obj._findUniqueId(translated)
            if translated is None:
                return
        langs = self.get_langs(self.storage[lang][id])
        if len(langs) == 1 and langs[0] == lang and self.storage[lang][id] in self.storage['index']:
            del self.storage['index'][self.storage[lang][id]]
        self.storage[lang][id] = translated
        self.storage['index'][translated] = id
        self.dispatchEvent(id)

    def remove_translated_ids(self, id, event=True):
        """ Removes all registered translated ids for the given ID
        """
        id = safe_unicode(id)
        for lang in self.storage:
            if not lang == 'index' and id in self.storage[lang]:
                translated = self.storage[lang][id]
                if 'index' in self.storage and translated in self.storage['index']:
                    del self.storage['index'][translated]
                del self.storage[lang][id]
        if event:
            self.dispatchEvent(id)

    def get_translated_ids(self, id):
        """ Iterator of lang, ID pairs of all available translated IDs for the given ID
        """
        id = safe_unicode(id)
        if id in self.context:
            for lang in self.storage:
                if not lang == 'index' and id in self.storage[lang]:
                    yield lang, self.storage[lang][id]

    def get_translated_id(self, id, lang, event=True):
        """ Returns a translated ID of the object with the given ID and in the given language
        """
        id = safe_unicode(id)
        if not id in self.context:
            return id
        if not lang in self.storage:
            self.storage[lang] = OOBTree()
        if not id in self.storage[lang]:
            field = None
            try:
                obj = self.context[id]
                if IBaseObject.providedBy(obj) and (obj.isTemporary() or obj._isIDAutoGenerated(id)):
                    return id
                field = obj.Schema()['title']
                if not IMultilanguageField.providedBy(field):
                    return id

                field.setLanguage(lang)
                new_id = obj.generateNewId()
                field.resetLanguage()

                if new_id is None:
                    return id
                if (not 'index' in self.storage or
                    not new_id in self.storage['index'] or
                    not self.storage['index'][new_id] == id) and not new_id == id:
                    invalid_id = False
                    check_id = getattr(obj, 'check_id', None)
                    if check_id is not None:
                        invalid_id = check_id(new_id, required=1)

                    # If check_id told us no, or if it was not found, make sure we have an
                    # id unique in the parent folder.
                    if invalid_id:
                        unique_id = obj._findUniqueId(new_id)
                        if unique_id is not None:
                            if check_id is None or check_id(new_id, required=1):
                                new_id = unique_id
                                invalid_id = False

                    if invalid_id:
                        return id

                new_id = safe_unicode(new_id)
                self.storage[lang][id] = new_id
                if not 'index' in self.storage:
                    self.storage['index'] = OOBTree()
                self.storage['index'][new_id] = id
                if event:
                    self.dispatchEvent(id)
            except ConflictError:
                if field is not None and IMultilanguageField.providedBy(field):
                    field.resetLanguage()
                raise
            except:
                if field is not None and IMultilanguageField.providedBy(field):
                    field.resetLanguage()
                return id
        return self.storage[lang][id]

    def get_actual_id(self, translated):
        """ Returns the actual ID of the object linked with the given translated ID, None otherwise
        """
        translated = safe_unicode(translated)
        if 'index' in self.storage and translated in self.storage['index']:
            return self.storage['index'][translated]
        return None

    def get_object(self, id):
        """ Returns the object having the given translated ID if available, None otherwise
        """
        id = safe_unicode(id)
        if not 'index' in self.storage:
            self.storage['index'] = OOBTree()
        if id in self.storage['index']:
            return self.context[self.storage['index'][id]]
        return None

    def get_langs(self, id):
        """ Returns the languages of the given translated ID if available
        """
        id = safe_unicode(id)
        langs = []
        for lang in self.storage.keys():
            if not lang == 'index' and id in self.storage[lang].values():
                langs.append(lang)
        return langs

    def check_id(self, id):
        """ Whether the given ID is not already in use
        """
        id = safe_unicode(id)
        return not 'index' in self.storage or not id in self.storage['index']


def update_translated_ids(obj, event):
    if IBaseObject.providedBy(obj) and obj.isTemporary():
        return
    handler = IMultilanguageURLHandler(event.newParent, None)
    old_handler = IMultilanguageURLHandler(event.oldParent, None)
    if handler is not None and old_handler is not None:
        for lang, id in old_handler.get_translated_ids(event.oldName):
            handler.set_translated_id(event.newName, id, lang)
    if not event.newParent is event.oldParent and old_handler is not None:
        old_handler.remove_translated_ids(event.oldName, False)



## Python Script "object_rename"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=Show the rename form for an object
##

url = '%s/@@multilanguage_rename' % (context.absolute_url())

context.REQUEST.RESPONSE.redirect(url)



from BTrees.OOBTree import OOBTree
from BTrees.IOBTree import IOBTree

from AccessControl.SecurityManagement import getSecurityManager
from AccessControl.ZopeGuards import guarded_getattr
from Acquisition import aq_inner, aq_parent, aq_acquire, aq_base
from Acquisition.interfaces import IAcquirer
from zExceptions import NotFound
from OFS.Traversable import path2url, _marker
from OFS.interfaces import ITraversable
from ZODB.POSException import ConflictError

from Products.CMFCore.interfaces import ICatalogTool
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.utils import safe_callable
from Products.CMFPlone.interfaces import IPloneSiteRoot

from zope.interface import Interface
from zope.traversing.interfaces import TraversalError
from zope.traversing.namespace import namespaceLookup
from zope.traversing.namespace import nsParse
from zope.component import queryMultiAdapter

from raptus.multilanguageurls.interfaces import IMultilanguageURLHandler


# Patching OFS.Traversable.Traversable

def _getTranslatedPhysicalPath(obj, lang=None, event=True):
    if not hasattr(obj, 'getId'):
        return ()
    id = obj.getId()
    p = aq_parent(aq_inner(obj))
    handler = IMultilanguageURLHandler(p, None)
    if handler is not None:
        if lang is None:
            lang = getToolByName(obj, 'portal_languages').getPreferredLanguage()
        id = handler.get_translated_id(id, lang, event)

    path = (id,)

    if p is not None:
        path = _getTranslatedPhysicalPath(p, lang, event) + path

    return path

def absolute_url(self, relative=0):
    if relative:
        return self.virtual_url_path()
    spp = _getTranslatedPhysicalPath(self)
    try:
        toUrl = aq_acquire(self, 'REQUEST').physicalPathToURL
    except AttributeError:
        return path2url(spp[1:])
    return toUrl(spp)

def absolute_url_path(self):
    spp = _getTranslatedPhysicalPath(self)
    try:
        toUrl = aq_acquire(self, 'REQUEST').physicalPathToURL
    except AttributeError:
        return path2url(spp) or '/'
    return toUrl(spp, relative=1) or '/'

def virtual_url_path(self):
    spp = _getTranslatedPhysicalPath(self)
    try:
        toVirt = aq_acquire(self, 'REQUEST').physicalPathToVirtualPath
    except AttributeError:
        return path2url(spp[1:])
    return path2url(toVirt(spp))

def unrestrictedTraverse(self, path, default=_marker, restricted=False):
    """ Need to patch traversing to respect multilanguage urls
        
        (This is a direct copy from OFS.Traversable.Traversable, with only
        the marked parts adjusted)
    """
    from webdav.NullResource import NullResource
    if not path:
        return self

    if isinstance(path, str):
        # Unicode paths are not allowed
        path = path.split('/')
    else:
        path = list(path)

    REQUEST = {'TraversalRequestNameStack': path}
    path.reverse()
    path_pop = path.pop

    if len(path) > 1 and not path[0]:
        # Remove trailing slash
        path_pop(0)

    if restricted:
        validate = getSecurityManager().validate

    if not path[-1]:
        # If the path starts with an empty string, go to the root first.
        path_pop()
        obj = self.getPhysicalRoot()
        if restricted:
            validate(None, None, None, obj) # may raise Unauthorized
    else:
        obj = self

    resource = _marker
    try:
        while path:
            name = path_pop()

            # Starting patch (looking up the possible actual name if the name is a
            # translated one)
            handler = IMultilanguageURLHandler(obj, None)
            if handler is not None:
                actual = handler.get_actual_id(name)
                if actual is not None:
                    name = str(actual)
            # End of patch

            __traceback_info__ = path, name

            if name[0] == '_':
                # Never allowed in a URL.
                raise NotFound, name

            if name == '..':
                next = aq_parent(obj)
                if next is not None:
                    if restricted and not validate(obj, obj, name, next):
                        raise Unauthorized(name)
                    obj = next
                    continue

            bobo_traverse = getattr(obj, '__bobo_traverse__', None)
            try:
                if name and name[:1] in '@+' and name != '+' and nsParse(name)[1]:
                    # Process URI segment parameters.
                    ns, nm = nsParse(name)
                    try:
                        next = namespaceLookup(
                            ns, nm, obj, aq_acquire(self, 'REQUEST'))
                        if IAcquirer.providedBy(next):
                            next = next.__of__(obj)
                        if restricted and not validate(
                            obj, obj, name, next):
                            raise Unauthorized(name)
                    except TraversalError:
                        raise AttributeError(name)

                elif bobo_traverse is not None:
                    next = bobo_traverse(REQUEST, name)
                    if restricted:
                        if aq_base(next) is not next:
                            # The object is wrapped, so the acquisition
                            # context is the container.
                            container = aq_parent(aq_inner(next))
                        elif getattr(next, 'im_self', None) is not None:
                            # Bound method, the bound instance
                            # is the container
                            container = next.im_self
                        elif getattr(aq_base(obj), name, _marker) is next:
                            # Unwrapped direct attribute of the object so
                            # object is the container
                            container = obj
                        else:
                            # Can't determine container
                            container = None
                        # If next is a simple unwrapped property, its
                        # parentage is indeterminate, but it may have
                        # been acquired safely. In this case validate
                        # will raise an error, and we can explicitly
                        # check that our value was acquired safely.
                        try:
                            ok = validate(obj, container, name, next)
                        except Unauthorized:
                            ok = False
                        if not ok:
                            if (container is not None or
                                guarded_getattr(obj, name, _marker)
                                    is not next):
                                raise Unauthorized(name)
                else:
                    if getattr(aq_base(obj), name, _marker) is not _marker:
                        if restricted:
                            next = guarded_getattr(obj, name)
                        else:
                            next = getattr(obj, name)
                    else:
                        try:
                            next = obj[name]
                            # The item lookup may return a NullResource,
                            # if this is the case we save it and return it
                            # if all other lookups fail.
                            if isinstance(next, NullResource):
                                resource = next
                                raise KeyError(name)
                        except AttributeError:
                            # Raise NotFound for easier debugging
                            # instead of AttributeError: __getitem__
                            raise NotFound(name)
                        if restricted and not validate(
                            obj, obj, None, next):
                            raise Unauthorized(name)

            except (AttributeError, NotFound, KeyError), e:
                # Try to look for a view
                next = queryMultiAdapter((obj, aq_acquire(self, 'REQUEST')),
                                         Interface, name)

                if next is not None:
                    if IAcquirer.providedBy(next):
                        next = next.__of__(obj)
                    if restricted and not validate(obj, obj, name, next):
                        raise Unauthorized(name)
                elif bobo_traverse is not None:
                    # Attribute lookup should not be done after
                    # __bobo_traverse__:
                    raise e
                else:
                    # No view, try acquired attributes
                    try:
                        if restricted:
                            next = guarded_getattr(obj, name, _marker)
                        else:
                            next = getattr(obj, name, _marker)
                    except AttributeError:
                        raise e
                    if next is _marker:
                        # If we have a NullResource from earlier use it.
                        next = resource
                        if next is _marker:
                            # Nothing found re-raise error
                            raise e

            obj = next

        return obj

    except ConflictError:
        raise
    except:
        if default is not _marker:
            return default
        else:
            raise


# Patching OFS.ObjectManager.ObjectManager
from OFS.ObjectManager import ObjectManager

def checkValidId(self, id):
    handler = IMultilanguageURLHandler(self, None)
    if handler is not None and not handler.check_id(id):
        raise KeyError('The given id %s is already in use' % id)
ObjectManager.checkValidId = checkValidId


# Patching Products.ZCatalog.Catalog.Catalog
from Products.ZCatalog.Catalog import Catalog

__old_catalogObject = Catalog.catalogObject

def catalogObject(self, object, uid, threshold=None, idxs=None,
                  update_metadata=1):
    total = __old_catalogObject(self, object, uid, threshold, idxs,
                                update_metadata)
    self.updateTanslatedPaths(object, uid)
    return total

def updateTanslatedPaths(self, object, uid):
    if ICatalogTool.providedBy(object):
        return
    try:
        request = aq_acquire(object, 'REQUEST')
    except:
        return
    if request is not None:
        if not hasattr(self, 'translated_paths'):
            self.translated_paths = OOBTree()
        index = self.uids.get(uid, None)
        if index is None:
            return
        langs = getToolByName(object, 'portal_languages').getSupportedLanguages()
        for lang in langs:
            if not lang in self.translated_paths:
                self.translated_paths[lang] = IOBTree()
            self.translated_paths[lang][index] = str('/'.join(_getTranslatedPhysicalPath(object, lang, False)))
Catalog.updateTanslatedPaths = updateTanslatedPaths

def update_translated_paths(obj, event):
    catalog = getToolByName(obj, 'portal_catalog')
    def updateTranslatedPaths(obj, path=None):
        if (base_hasattr(obj, 'indexObject') and
            safe_callable(obj.indexObject) and
            base_hasattr(obj, 'getPhysicalPath') and
            safe_callable(obj.getPhysicalPath)):
            try:
                catalog._catalog.updateTanslatedPaths(obj, '/'.join(obj.getPhysicalPath()))
            except:
                pass
    portal = aq_parent(aq_inner(catalog))
    updateTranslatedPaths(obj)
    portal.ZopeFindAndApply(obj, search_sub=True, apply_func=updateTranslatedPaths)

__old_uncatalogObject = Catalog.uncatalogObject

def uncatalogObject(self, uid):
    __old_uncatalogObject(self, uid)
    index = self.uids.get(uid, None)
    if index is None:
        return
    if not hasattr(self, 'translated_paths'):
        return
    for lang in self.translated_paths:
        if index in self.translated_paths[lang]:
            del self.translated_paths[lang][index]


# Patching Products.ZCatalog.CatalogBrains.AbstractCatalogBrain
from Products.ZCatalog.CatalogBrains import AbstractCatalogBrain

__old_getURL = AbstractCatalogBrain.getURL

def getURL(self, relative=0):
    if not hasattr(self.aq_parent._catalog, 'translated_paths'):
        return __old_getURL(self, relative)
    lang = getToolByName(self.aq_parent, 'portal_languages').getPreferredLanguage()
    if (not lang in self.aq_parent._catalog.translated_paths or
        not self.data_record_id_ in self.aq_parent._catalog.translated_paths[lang]):
        return __old_getURL(self, relative)
    return self.REQUEST.physicalPathToURL(self.aq_parent._catalog.translated_paths[lang][self.data_record_id_], relative)



from Acquisition import aq_parent, aq_inner, aq_acquire
from AccessControl.Permissions import copy_or_move

from zope import interface
from zope import schema
from zope.component import adapts
from zope.formlib.form import EditForm, FormFields, action, applyData
from zope.schema.interfaces import ValidationError
from zope.app.form.interfaces import WidgetInputError
from plone.app.imaging.interfaces import IBaseObject

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.permissions import ModifyPortalContent, DeleteObjects
from Products.CMFPlone import PloneMessageFactory as _p
from Products.CMFPlone.utils import safe_unicode
from Products.statusmessages.interfaces import IStatusMessage

from raptus.multilanguagefields.interfaces import IMultilanguageField
from raptus.multilanguageurls.interfaces import IMultilanguageURLHandler


class InvalidIDError(ValidationError):
    def __init__(self, doc):
        self.__doc__ = doc
    def doc(self):
        return self.__doc__


class IRenameForm(interface.Interface):
    """ The rename form
    """

    id = schema.TextLine(
        title=_p(u'label_new_short_name', default=u'New Short Name'),
        description=_p(u'help_short_name_url', default=u'Short name is the part that shows up in the URL of the item.'),
        required=True
    )

    title = schema.TextLine(
        title=_p(u'label_new_title', u'New Title'),
        required=True
    )


class RenameFormAdapter(object):
    interface.implements(IRenameForm)
    adapts(IBaseObject)

    def __init__(self, context):
        self.context = context
        self.languages = getToolByName(context, 'portal_languages').getSupportedLanguages()
        self.handler = IMultilanguageURLHandler(aq_parent(aq_inner(context)), None)

    def __setattr__(self, name, value):
        if name in ('languages', 'context', 'handler',):
            object.__setattr__(self, name, value)
        elif name == 'id':
            self.context.setId(value)
        elif name == 'title':
            self.context.setTitle(value)
        elif name in self.languages:
            self.handler.set_translated_id(self.id, value, name)
        else:
            object.__setattr__(self, name, value)

    def __getattr__(self, name):
        if name in ('languages', 'context', 'handler',):
            return object.__getattr__(self, name)
        if name == 'id':
            return self.context.getId()
        if name == 'title':
            return self.context.Title()
        if name in self.languages:
            return self.handler.get_translated_id(self.id, name)
        return object.__getattr__(self, name)


class RenameForm(EditForm):

    label = _p(u'heading_rename_item', default=u'Rename item')
    description = _p(u'description_rename_item',
                    default=u'Each item has a Short Name and a Title, which you can change '
                             'by entering the new details below.')

    def __init__(self, context, request):
        super(RenameForm, self).__init__(context, request)
        for name, value in self.request.form.items():
            self.request.form[name] = safe_unicode(value)
        self.form_fields = FormFields(IRenameForm)
        self.form_name = '%s (%s)' % (self.context.Title(), self.context.getId())
        mtool = getToolByName(self.context, 'portal_membership')
        title = self.context.Schema()['title']
        self.id = self.context.getId()
        self.languages = {}
        if (IMultilanguageField.providedBy(title) or
            not mtool.checkPermission(DeleteObjects, self.context) or
            not mtool.checkPermission(copy_or_move, self.context)):
            self.form_fields = self.form_fields.omit('id')
        if (IMultilanguageField.providedBy(title) or
            not mtool.checkPermission(ModifyPortalContent, self.context)):
            self.form_fields = self.form_fields.omit('title')
        if IMultilanguageField.providedBy(title):
            self.handler = IMultilanguageURLHandler(aq_parent(aq_inner(self.context)), None)
            for lang in title.getAvailableLanguages(self.context):
                self.languages[lang['name']] = lang['title']
                self.form_fields = self.form_fields + FormFields(
                    schema.TextLine(
                        __name__=lang['name'],
                        title=lang['title'],
                        default=self.handler.get_translated_id(self.id, lang['name']),
                        required=True
                    )
                )
                self.form_fields[lang['name']].interface = IRenameForm

    def validate(self, action, data):
        errors = super(RenameForm, self).validate(action, data)
        for field in self.form_fields:
            name = field.field.getName()
            if name == 'title':
                continue
            value = data.get(name, u'')
            check_id = getattr(self.context, 'check_id', None)
            if ((name == 'id' and not value == self.id) or
                (name in self.languages.keys() and not value == self.handler.get_translated_id(self.id, name) and
                 not value == self.id and not self.id == self.handler.get_actual_id(value)) and
                check_id is not None):
                error = check_id(str(value), required=1)
                if error:
                    widget = self.widgets[name]
                    widget._error = WidgetInputError(widget.name, widget.label, InvalidIDError(error))
                    errors.append(widget._error)
        return errors

    @action(_p(u'label_rename_all', default=u'Rename All'), name=u'rename')
    def handle_rename_action(self, action, data):
        context = aq_inner(self.context)
        oldid = context.getId()
        oldtranslatedids = self.handler.get_translated_ids(oldid)
        descriptions = applyData(context, self.form_fields, data, self.adapters)
        statusmessage = IStatusMessage(self.request)
        if descriptions:
            statusmessage.addStatusMessage(_p(u'Changes saved.'), u'success');
        else:
            statusmessage.addStatusMessage(_p(u'No changes made.'));
        self.request.RESPONSE.redirect(context.absolute_url())



from zope.interface import implements
from zope.component import adapts
from zope.publisher.interfaces import IPublishTraverse, NotFound
from zope.publisher.interfaces.http import IHTTPRequest
from zope.annotation.interfaces import IAnnotatable
from plone.app.imaging.interfaces import IBaseObject
from plone.app.imaging.traverse import ImageTraverser

from Products.CMFCore.utils import getToolByName
from ZPublisher.BaseRequest import DefaultPublishTraverse

from raptus.multilanguageurls.interfaces import IMultilanguageURLHandler


class MultilanguageTraverse(object):
    implements(IPublishTraverse)
    adapts(IAnnotatable, IHTTPRequest)

    def __init__(self,context,request):
        self.context = context
        self.request = request
        self.default_traverse = DefaultPublishTraverse(context, request)
        self.handler = IMultilanguageURLHandler(context, None)

    def publishTraverse(self, request, name):
        if self.handler is not None:
            ob = self.handler.get_object(name)
            if ob is not None:
                langs = self.handler.get_langs(name)
                if len(langs) == 1:
                    try:
                        portal_languages = getToolByName(self.context, 'portal_languages')
                        portal_languages.REQUEST.set_lazy('set_language', lambda: langs[0])
                        portal_languages.setLanguageBindings()
                    except:
                        pass
                return ob
        try:
            return self.default_traverse.publishTraverse(request, name)
        except NotFound:
            pass
        except KeyError:
            pass
        except AttributeError:
            pass
        raise NotFound(self.context, name, self.request)


class MultilanguageImageTraverse(MultilanguageTraverse):
    implements(IPublishTraverse)
    adapts(IBaseObject, IHTTPRequest)

    def publishTraverse(self, request, name):
        try:
            return super(MultilanguageImageTraverse, self).publishTraverse(request, name)
        except NotFound:
            pass
        return ImageTraverser(self.context, self.request).publishTraverse(request, name)
